@@ -9,18 +9,15 @@ n <- ncol(x)
 
     for(i in 1 : (n - 1)){
         indiv.causes[[i]] <- sapply((i + 1) : n, function(b) NNS.caus(x[ , i], x[ , b], plot = FALSE, tau = tau))
-    }
-
-    for(i in 1 : length(indiv.causes)){
-        indiv.causes[[i]] <-  (abs(indiv.causes[[i]][2, ]) - abs(indiv.causes[[i]][1, ]))
+        indiv.causes[[i]] <- (abs(indiv.causes[[i]][2, ]) - abs(indiv.causes[[i]][1, ]))
     }
 
     causes <- matrix(NA, n, n)
     causes[lower.tri(causes, diag = FALSE)] <- unlist(indiv.causes)
-
+    causes[upper.tri(causes, diag = FALSE)] <- -t(causes)[upper.tri(causes)]
 
     diag(causes) <- 1
-    causes <- pmax(causes, t(-causes), na.rm = TRUE)
+
     colnames(causes) <- colnames(x)
     rownames(causes) <- colnames(x)
 
